@@ -1,7 +1,7 @@
 package Dancer::Test;
 our $AUTHORITY = 'cpan:SUKRIA';
 #ABSTRACT: Test helpers to test a Dancer application
-$Dancer::Test::VERSION = '1.3141'; # TRIAL
+$Dancer::Test::VERSION = '1.3143'; # TRIAL
 # test helpers for Dancer apps
 
 use strict;
@@ -314,7 +314,7 @@ sub dancer_response {
         elsif ( $args->{body} ) {
             $content      = $args->{body};
             $content_type = $args->{content_type}
-                || 'application/x-www-form-urlencoded';
+                || 'text/plain';
 
             # coerce hashref into an url-encoded string
             if ( ref($content) && ( ref($content) eq 'HASH' ) ) {
@@ -325,6 +325,7 @@ sub dancer_response {
                     push @tokens, "${name}=${value}";
                 }
                 $content = join( '&', @tokens );
+                $content_type = 'application/x-www-form-urlencoded';
             }
         }
         elsif ( $args->{files} ) {
@@ -454,7 +455,7 @@ Dancer::Test - Test helpers to test a Dancer application
 
 =head1 VERSION
 
-version 1.3141
+version 1.3143
 
 =head1 SYNOPSIS
 
